import Logger, { LoggerType } from '@/application/logger'
import type { UUID } from '@/application/types'
import { Map } from '@/entities/map'
import LoadedMap from '@/models/loadedMap'
import MapCharacter from '@/models/mapCharacter'
import MapRepository from '@/repositories/mapRepository'

class MapManager {
  private readonly maps: Record<UUID, LoadedMap> = {}
  private logger = Logger.type(LoggerType.GAME)

  public async boot(): Promise<void> {
    const mapRepository = new MapRepository()
    const maps = await mapRepository.getAll()
    await mapRepository.getEntityManager().populate(maps, mapRepository.POPULATE_ALL as never[])

    await Promise.all(maps.map((map) => this.loadMap(map)))
    this.logger.info(`Map manager loaded with ${Object.keys(this.maps).length} maps`)
  }

  public async loadMap(map: Map): Promise<LoadedMap> {
    const loadedMap = new LoadedMap(map)
    this.maps[map.id] = loadedMap
    this.logger.info(`Map ID ${map.id} loaded`)
    return loadedMap
  }

  public unloadMap(mapId: UUID): void {
    const map = this.maps[mapId]
    if (!map) return

    delete this.maps[mapId]
    this.logger.info(`Map ID ${mapId} unloaded`)
  }

  public getLoadedMaps(): LoadedMap[] {
    return Object.values(this.maps)
  }

  public getMapById(mapId: UUID): LoadedMap | undefined {
    return this.maps[mapId]
  }

  public getCharacterById(characterId: UUID): MapCharacter | null {
    for (const map of Object.values(this.maps)) {
      const character = map.getCharacterById(characterId)
      if (character) return character
    }
    return null
  }
}

export default new MapManager()
